% x = btk_route(CM, s, d)
% "btk_route" computes all the paths connecting "s" to "d".
% "CM" means connectivity matrix. Each row of CMs in this program have 3 elements. They may be 2 complex numbers each indicating an atom in substrate and the corresponding atom in product, and an integral number indicating the responsible enzyme reaction (reaction number). In a complex number, m + c*i, indicating an atom, m is a metabolite number, and c is an intra-metabolite atom number. Elements of CMs maybe also be integral numbers indicating nodes or edges in general networks.
% "s" and "d" may be complex numbers or integral numbers.
% "x" is a cell array consisting of row number sequences.
% The row number sequences correspond to the above paths between "s" to "d".
% March 7, 2017: A bug was fixed. This bug had caused a problem when this function did not find any paths.
% July 6, 2017: Explanation in this file was enhanced. Bugs related to "error" command were fixed.
function x = btk_route(CM, s, d);
if ( nargin == 1 )
     s=input('start point=?');
     d=input('destination=?');
elseif ( nargin == 2 )
     error('Check nargin.');
elseif ( nargin == 0 )
     error('Check nargin.');
elseif ( nargin >= 4 )
     error('Check nargin.');
end
x = cell(0,0);
v = find( CM(:,1) == s );
if ( isempty(v) == 0 )
     vs = v( find( CM(v,2) ~= d ), 1);
     vd = v( find( CM(v,2) == d ), 1);
     i0 = 1;
     if ( isempty(vd) == 0 )
           [vdr, vdc] = size(vd);
           while ( i0 <= vdr )
                 x{i0,1} = vd(i0,1);
                 i0 = i0 + 1;
           end
     end
     PO = vs;
     if ( isempty(PO) == 0 )
           while (1)
                 [POr, POc] = size(PO);
                 i1 = 1;
                 vpn = [];
                 while ( i1 <= POr )
                       v0 = find( CM(PO(i1,POc),2) == CM(:,1) );
                       [v0r, v0c] = size(v0);
                       v0i = 1;
                       while ( v0i <= v0r )
                           if ~( any( CM(PO(i1,:),1) == CM(v0(v0i,1),1) ) );
                               vpn = [ vpn; i1, v0(v0i,1) ];
                           end
                           v0i = v0i + 1;
                       end
                       i1 = i1 + 1;
                 end
                 if ( isempty(vpn) == 0 )
                       vs = find( CM(vpn(:,2),2) ~= d );
                       vd = find( CM(vpn(:,2),2) == d );
                       if ( isempty(vd) == 0 )
                             [vdr, vdc] = size(vd);
                             vdi = 1;
                             while ( vdi <= vdr )
                                   x{i0,1} = [ PO(vpn(vd(vdi,1),1),:), vpn(vd(vdi,1),2) ];
                                   i0 = i0 + 1;
                                   vdi = vdi + 1;
                             end 
                       end
                       if ( isempty(vs) == 0 )
                             PO = [ PO(vpn(vs,1),:), vpn(vs,2) ];
                       else
                             PO = [];
                       end
                 else
                       PO = [];
                 end
                 if ( isempty(PO) == 1 )
                       break;
                 end
           end
     end
end