% [tfv,n23,nr2,n12,n13,n14,n33] = print_cycle_rxn( x, pn, CM, name )
% print_cycle_rxn prints use of citric acid cycle (CAC) and pentose phosphate pathway (PPP) as a cycle, which is judged from repeated use of citrate synthase and glucose-6-phosphate dehydrogenase (G6PD), and multiple use of D-fructose 6-phosphate or D-glyceraldehyde 3-phosphate without repetition of G6PD reaction.
% As for use of citric acid cycle (CAC) and pentose phosphate pathway (PPP) as a cycle, 
% tfv == 0 means none of CAC and PPP.
% tfv == 1 means PPP only.
% tfv == 2 means CAC only.
% tfv == 3 means both of CAC and PPP.
% n23,nr2,n12,n13,n14,and n33 mean how many times the following reactions are used.
% n23: Glucose-6-phosphate dehydrogenase, G6PD
% nr2: Phosphohexose isomerase (reverse)
% n12: Pyruvate dehydrogenase complex
% n13: Pyruvate carboxylase
% n14: Citrate synthase
% n33: Phosphoenolpyruvate carboxykinase
% x is a cell consisting of row number sequences of connectivity matrices (CMs). Each row of CMs in this program have 3 elements which are 2 complex numbers each indicating an atom in substrate and the corresponding atom in product, and an integral number indicating the responsible enzyme reaction (reaction number). In a complex number, m + c*i, indicating an atom, m is a metabolite number, and c is an intra-metabolite atom number. rxnnic in this function is column number for reaction number in connectivity matrix.
% pn is a path number indicating a specific sequence described in x.
% name: name of result file
function [tfv,n23,nr2,n12,n13,n14,n33] = print_cycle_rxn( x, pn, CM, name );
rxnnic = 3;
tfv = 0;
CM_row_number_seq = x{pn,1};
y = CM(CM_row_number_seq,rxnnic);
[n23,c] = size(find(y == 23));
[nr2,c] = size(find(y == -2));
[n12,c] = size(find(y == 12));
[n13,c] = size(find(y == 13));
[n14,c] = size(find(y == 14));
[n33,c] = size(find(y == 33));
fid = fopen(name,'a');
fprintf(fid,'%s\n',['Pathway No. ', int2str(pn)]);
fprintf(fid,'%s\n',['Cycle(s) confirmed by reactions used multiple times:']);
if ( n23 >= 2 )
  fprintf(fid,'%s\n', ['   Pentose phosphate pathway']);
  tfv = tfv + 1;
end
if ( n14 >= 2 )
  fprintf(fid,'%s\n', ['   Citric acid cycle']);
  tfv = tfv + 2;
end
if (( n23 < 2 )&( n14 < 2 ))
  fprintf(fid,'%s\n', ['   None']);
end
fprintf(fid,'%s\n',['Number of appearance of several reactions']);
fprintf(fid,'%s\n',['   Glucose-6-phosphate dehydrogenase, G6PD: ',int2str(n23)]);
fprintf(fid,'%s\n',['   Phosphohexose isomerase (reverse):       ',int2str(nr2)]);
fprintf(fid,'%s\n',['   Pyruvate dehydrogenase complex:          ',int2str(n12)]);
fprintf(fid,'%s\n',['   Pyruvate carboxylase:                    ',int2str(n13)]);
fprintf(fid,'%s\n',['   Citrate synthase:                        ',int2str(n14)]);
fprintf(fid,'%s\n',['   Phosphoenolpyruvate carboxykinase:       ',int2str(n33)]);
[rncr,rnsc] = size(CM_row_number_seq);
z = [CM(CM_row_number_seq,1);CM(CM_row_number_seq(1,rnsc),2)];
rz = real(z);
[m3,c] = size(find(rz == 3));
[m5,c] = size(find(rz == 5));
fprintf(fid,'%s\n',['D-Fructose 6-phosphate used multiple times without repetition of G6PD reaction:']);
if ( m3 >= 2 )&(( tfv == 2 )|( tfv == 0 ))
  fprintf(fid,'%s\n', ['   Exist']);
else
  fprintf(fid,'%s\n', ['   Not exist']);
end
fprintf(fid,'%s\n',['D-Glyceraldehyde 3-phosphate used multiple times without repetition of G6PD reaction:']);
if ( m5 >= 2 )&(( tfv == 2 )|( tfv == 0 ))
  fprintf(fid,'%s\n', ['   Exist']);
else
  fprintf(fid,'%s\n', ['   Not exist']);
end
fprintf(fid,'%s\n', '');
fclose(fid);
